import { AppContext, ExtractPropTypes, Component, Ref, App } from 'vue-demi';

interface MountOptions {
    /** the dom node to mount at render time */
    root?: HTMLElement;
    /** current app context */
    appContext?: AppContext;
}

interface ImperativePromise<T = any> extends Promise<T> {
    cancel: Function;
    confirm: Function;
}
interface ImperativeOverlay<Props, Resolved> {
    (props?: ExtractPropTypes<Props>, options?: MountOptions): ImperativePromise<Resolved>;
}
interface RenderOptions<Props = unknown> extends MountOptions {
    props?: ExtractPropTypes<Props>;
}
/**
 * Create imperative overlay
 * @param component Component
 */
declare function createOverlay<Props, Resolved = void>(component: Component): ImperativeOverlay<Props, Resolved>;
/**
 * Execute overlay component
 * @param component Component
 * @param options mount options and props
 */
declare function renderOverlay<Props = {}, Resolved = void>(component: Component, options?: RenderOptions<Props>): ImperativePromise<Resolved>;

interface OverlayMeta {
    /** the notification cancel, modify visible, and destroy it after the animation ends */
    cancel: Function;
    /** the notification confirm, modify visible, and destroy it after the animation ends */
    confirm: Function;
    /** destroy the current instance (immediately) */
    vanish: Function;
    /** visible control popup display and hide */
    visible: Ref<boolean>;
    /** use in template */
    isTemplate?: boolean;
}

interface OverlayMetaOptions {
    /** animation duration to avoid premature destruction of components */
    animation?: number;
    /** whether to set visible to true immediately */
    immediate?: boolean;
    /**
     * v-model fields used by template
     *
     * @default 'visible'
     */
    model?: string;
    /**
     * cancel event name used by the template
     *
     * @default 'cancel'
     */
    cancel?: string;
    /**
     * confirm event name used by the template
     *
     * @default 'confirm'
     */
    confirm?: string;
    /**
     * whether to automatically handle components based on visible and animation
     *
     * @default true
     */
    automatic?: boolean;
}
/**
 * get overlay layer meta information
 * @function cancel  the notification cancel, modify visible, and destroy it after the animation ends
 * @function confirm the notification confirm, modify visible, and destroy it after the animation ends
 * @function vanish destroy the current instance (immediately)
 * @field vnode the VNode of the current injection layer
 * @field visible control popup display and hide
 * @returns
 */
declare function useOverlayMeta(options?: OverlayMetaOptions): OverlayMeta;

declare const install: (app: App) => void;
declare const unoverlay: {
    install: (app: App) => void;
};

export { ImperativeOverlay, OverlayMetaOptions, RenderOptions, createOverlay, unoverlay as default, install, renderOverlay, useOverlayMeta };
