"use strict";

// src/internal/index.ts
var context = {
  appContext: null
};
var OverlayMetaKey = Symbol("__imperative_overlay_key");

// src/hooks/meta.ts
import { getCurrentInstance, inject, onMounted, provide, ref, watch } from "vue-demi";
import { useVModel } from "@vueuse/core";

// src/utils/index.ts
function delay(milliseconds) {
  return new Promise((resolve) => setTimeout(resolve, milliseconds));
}
function createPromiser() {
  let resolve;
  let reject;
  const promise = new Promise((_resolve, _reject) => {
    resolve = _resolve;
    reject = _reject;
  });
  return { promise, reject, resolve };
}
function noop() {
}
function allowed() {
  throw new Error("overlay - Error: It is not allowed to call confirm and cancel externally immediately, please wait for the component to render");
}
var portalIndex = 1;
function createGlobalNode(id, root = document.body) {
  const el = document.createElement("div");
  el.id = `${id}--${portalIndex++}`;
  root.appendChild(el);
  return el;
}

// src/hooks/meta.ts
function useOverlayMeta(options = {}) {
  const { animation = 0, immediate = true, model = "visible", automatic = true } = options;
  const defaultMeta = getTemplateMeta(model, options);
  const meta = inject(OverlayMetaKey, defaultMeta) || defaultMeta;
  if (!meta.isTemplate && automatic) {
    watch(meta.visible, async () => {
      var _a;
      if (meta.visible.value)
        return void 0;
      if (animation > 0)
        await delay(animation);
      (_a = meta.vanish) == null ? void 0 : _a.call(meta);
    });
  }
  if (immediate)
    onMounted(() => meta.visible.value = true);
  provide(OverlayMetaKey, null);
  return meta;
}
function getTemplateMeta(model, options = {}) {
  const instance = getCurrentInstance();
  const visible = instance ? useVModel(instance.props, model) : ref(false);
  const cancel = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit(options.cancel || "cancel", value);
  };
  const confirm = (value) => {
    visible.value = false;
    instance == null ? void 0 : instance.emit(options.confirm || "confirm", value);
  };
  return {
    cancel,
    confirm,
    vanish: noop,
    visible,
    isTemplate: true
  };
}

// src/hooks/visible.ts
import { ref as ref2 } from "vue";
function useVisible(promiser, _vanish) {
  const { reject, resolve, promise } = promiser;
  const visible = ref2(false);
  function cancel(value) {
    reject == null ? void 0 : reject(value);
    visible.value = false;
  }
  function confirm(value) {
    resolve == null ? void 0 : resolve(value);
    visible.value = false;
  }
  function vanish() {
    _vanish == null ? void 0 : _vanish();
    reject == null ? void 0 : reject();
  }
  promise.confirm = confirm;
  promise.cancel = cancel;
  return { visible, confirm, cancel, vanish };
}

// src/transform/index.ts
import { provide as provide2 } from "vue-demi";

// src/helper/render.ts
import { createApp, createVNode, defineComponent, h, render } from "vue-demi";
var renderInstance = (component, props, options = {}) => {
  const vanish = () => {
    render(null, container);
  };
  const name = `${component.name}OverlayProvider`;
  const Provider = defineComponent({
    name,
    setup() {
      var _a;
      (_a = options.setup) == null ? void 0 : _a.call(options);
    },
    render() {
      return h(component, props);
    }
  });
  const container = createGlobalNode(name);
  const vnode = createVNode(Provider);
  vnode.appContext = options.appContext || context.appContext;
  render(vnode, container);
  if (container.firstElementChild) {
    const root = options.root || document.body;
    root.append(container.firstElementChild);
  }
  return { vanish, vnode };
};

// src/transform/index.ts
function createOverlay(component) {
  function executor(props, promiser, options) {
    let vanish;
    function setup() {
      provide2(OverlayMetaKey, useVisible(promiser, vanish));
    }
    ({ vanish } = renderInstance(component, props, { ...options, setup }));
  }
  function caller(props, options) {
    const promiser = createPromiser();
    promiser.promise.confirm = allowed;
    promiser.promise.cancel = allowed;
    executor(props, promiser, options);
    return promiser.promise;
  }
  return caller;
}
function renderOverlay(component, options = {}) {
  return createOverlay(component)(options.props, options);
}

// src/index.ts
var install = (app) => {
  context.appContext = app._context;
};
var unoverlay = { install };
var src_default = unoverlay;
export {
  createOverlay,
  src_default as default,
  install,
  renderOverlay,
  useOverlayMeta
};
//# sourceMappingURL=index.esm.js.map
